/* REXX: simplified http://members.dyndns.org/nic.update? script, */
/* needs RxSock.DLL (REXX sockets library from IBM), a dynamic IP */
/* (i.e. a new dialup internet connection), and a DynDNS HOSTNAME */
/* not already associated with this IP.                           */

/* Usage: RXDYNDNS.CMD  interface  hostname  username  password   */
/* resp.: call RXDYNDNS interface, hostname, username, password   */
/*  e.g.: call RXDYNDNS 'ppp0','test.dnsalias.org','test','test'  */

/* New features in version 0.7 (0.8 is a Syslog message bug fix): */
/* You can specify multiple comma separated hosts, example line   */
/* RXDYNDNS.CMD ppp0 test.dnsalias.org,test.dyndns.org test test  */

/* You can't mix DynDNS and OrgDNS hosts, the relevant server is  */
/* derived from the last host, default members.dyndns.org:8245    */
/* - for last host *.orgdns.org server members.orgdns.org:80      */
/* - for tests use *localhost   server          localhost:80      */

/* You can add certain DynDNS parameters as fifth argument, e.g.: */
/* RXDYNDNS.CMD ppp0 test.dnsalias.org test test offline=YES      */
/* RXDYNDNS.CMD ppp0 test.dnsalias.org test test system=custom    */
/* RXDYNDNS.CMD ppp0 test.dnsalias.org test test mx=your.mx.test  */

/* Separate multiple additional parameters with '&' or semicolon  */
/* as in system=custom&offline=YES or system=custom;offline=YES   */

/* Internally commata, ampersands, semicolons, and even blanks    */
/* are accepted as separators, but in the resulting GET HTTP/1.0  */
/* query parameters are separated by ampersands and hostnames by  */
/* commata (resulting in parameter hostname=host1,host2 etc.).    */

/* Normally the script doesn't allow you to update a host, if its */
/* IP is already set.  Now the presence of parameter &offline=YES */
/* bypasses this restriction.   You MUST NOT specify &offline=NO, */
/* this feature is reserved for manual tests.  Only donators are  */
/* allowed to use &offline=YES, otherwise you'd get a "!donator". */

/* The error message "nohost" normally indicates an unknown host. */
/* In the case of parameter &system=custom resp. &system=statdns  */
/* "nohost" can also indicate a host not known in the specified   */
/* system.  Please note that &system=custom resp. &system=statdns */
/* are no free services.  RXDYNDNS won't work for &system=statdns */
/* - serious offers for an individual "statdns" solution welcome. */

/* The result is either an error message (incl. DynDNS messages)  */
/* or the message 'good a.b.c.d' after updating dyn. IP a.b.c.d . */

/* Please change the COPYLEFT below if you modify the script, see */
/* <URL:http://purl.net/xyzzy/rxdyndns.htm>, (c) 2003 F.Ellermann */

   signal on novalue  name TRAP  ;  signal on syntax name TRAP
   signal on failure  name TRAP  ;  signal on halt   name TRAP

   COPYLEFT = 'rxdyndns/0.8 (+http://purl.net/xyzzy/rxdyndns.htm)'
   PEERNAME = 'members.dyndns.org'     /* HTTP server for members */
   PEERPORT = 8245                     /* 80 works without proxy  */
   VARIABLE = '&wildcard=NOCHG'        /* always sent by RXDYNDNS */

   if arg() = 1                        /* i.e. if used as command */
      then parse arg IFCONFIG  HOSTNAME  USERNAME  PASSWORD  MORE
      else parse arg IFCONFIG, HOSTNAME, USERNAME, PASSWORD, MORE

   HOSTNAME = strip( HOSTNAME )  ;  CRLF  = x2c( 0D0A )
   USERNAME = strip( USERNAME )  ;  TSOCK = -1
   PASSWORD = strip( PASSWORD )  ;  EXPO  = 'CRLF TSOCK'

   N = 'SockLoadFuncs'                 /* load RxSock.DLL entries */
   if RxFuncQuery( N ) then if RxFuncAdd( N, 'RXSOCK', N )
      then return ABORT( N 'in RXSOCK.DLL not found' )
   call SockLoadFuncs 'N'              /* always to verify RxSock */

   select                              /* no alpha: -h, /h, -?, ? */
      when datatype( IFCONFIG, 'A' ) = 0 then return USAGE()
      when strip( HOSTNAME USERNAME PASSWORD ) = '' then do
         HOSTNAME = 'test.dnsalias.org'
         USERNAME = 'test'       ;  PASSWORD = 'test'
      end                              /* DynDns domain for tests */
      when translate( right( HOSTNAME, 11 )) = '.ORGDNS.ORG' then do
         PEERPORT = 80           ;  PEERNAME = 'members.orgdns.org'
      end                              /* update OrgDNS.org host  */
      when translate( right( HOSTNAME,  9 )) = 'LOCALHOST' then do
         PEERPORT = 80           ;  PEERNAME = 'localhost'
      end                              /* test any localhost echo */
      when HOSTNAME = '' then return USAGE( 'missing arg. 2 host'  )
      when USERNAME = '' then return USAGE( 'missing arg. 3 login' )
      when PASSWORD = '' then return USAGE( 'missing arg. 4 sword' )
      otherwise   nop
   end

   if SockGetHostByName( PEERNAME, 'PEER.1' ) = 0
      then return ABORT( 'unknown server' PEERNAME )

   do while MORE <> ''                 /* check DynDNS parameters */
      parse var MORE TEXT MORE         /* blank or &,; separators */
      if MORE = ''   then parse var TEXT TEXT '&' MORE
      if MORE = ''   then parse var TEXT TEXT ';' MORE
      if MORE = ''   then parse var TEXT TEXT ',' MORE

      parse var TEXT TEXT '=' N  ;  TEXT = '&' || TEXT || '='
      if translate( N ) = 'YES' | translate( N ) = 'NO'
         then N = translate( N )       /* force upper case YES NO */
      if sign( pos( TEXT, VARIABLE ))  /* redefinitions not okay: */
         then return ABORT( 'parameter conflict' TEXT || N VARIABLE )

      TEXT = TEXT || N  ;  VARIABLE = VARIABLE || TEXT
      select
         when TEXT = '&system=dyndns'  then iterate
         when TEXT = '&system=custom'  then iterate
         when TEXT = '&offline=YES'    then iterate
         when TEXT = '&offline=NO'     then iterate
         when TEXT = '&backmx=YES'     then iterate
         when TEXT = '&backmx=NO'      then iterate
         when abbrev( TEXT, '&mx=' )   then do
            if SockGetHostByName( N, 'HOST.1' ) then iterate
            return ABORT( 'invalid' TEXT '(unknown MX' N || ')' )
         end
         otherwise   return ABORT( 'unsupported parameter' TEXT )
      end
   end

   PEER.1FAMILY   = 'AF_INET'    ;  HOSTLIST = ''
   PEER.1PORT     = PEERPORT     ;  MYIP = DYNIP( IFCONFIG )

   do until HOSTNAME = ''              /* a comma separated list, */
      parse var HOSTNAME MORE HOSTNAME /* (other separators okay) */
      if HOSTNAME = ''  then parse var MORE MORE '&' HOSTNAME
      if HOSTNAME = ''  then parse var MORE MORE ';' HOSTNAME
      if HOSTNAME = ''  then parse var MORE MORE ',' HOSTNAME

      if SockGetHostByName( MORE, 'HOST.1' ) = 0
         then return ABORT( 'unknown host' MORE )

      do N = 1 to HOST.1ADDR.0         /* check MORE host aliases */
         if MYIP = HOST.1ADDR.N then leave N
      end N
      if N > HOST.1ADDR.0 | sign( pos( '&offline=', VARIABLE ))
         then HOSTLIST = HOSTLIST || ',' || MORE
   end
   if HOSTLIST = ''  then return ABORT( MYIP 'already set' )

   TEXT = 'GET /nic/update?hostname='  || substr( HOSTLIST, 2 )
   TEXT = TEXT || VARIABLE || '&myip=' || MYIP 'HTTP/1.0'
   TEXT = TEXT || CRLF     || 'User-Agent:' COPYLEFT
   TEXT = TEXT || CRLF     || 'Host:'  || PEERNAME
   TEXT = TEXT || BASIC( USERNAME || ':' || PASSWORD )
   TEXT = TEXT || CRLF     || 'Pragma: no-cache'
   TEXT = TEXT || CRLF     || 'Connection: close'
   TEXT = TEXT || CRLF     || CRLF

   TSOCK = SockSocket( 'AF_INET', 'SOCK_STREAM', 'IPPROTO_TCP' )
   if TSOCK = -1 then return ABORT( 'SockSocket() =' TSOCK )

   signal on novalue  name FATAL ;  signal on syntax name FATAL
   signal on failure  name FATAL ;  signal on halt   name FATAL

   if SockConnect( TSOCK, 'PEER.1' ) <> 0 then
      return ABORT( 'SockConnect' )
   if SockSend( TSOCK, TEXT ) = -1 then
      return ABORT( 'SockSend' )
   if SockShutDown( TSOCK, 1 ) <> 0 then
      return ABORT( 'SockShutDown' )

   TEXT = ''
   do until N = 0
      N = SockRecv( TSOCK, 'MORE', 1024 )
      if N = -1 then return ABORT( 'SockRecv' )
      TEXT = TEXT || left( MORE, N )
   end

   if SockClose( TSOCK ) <> 0 then return ABORT( 'SockClose' )
   signal on novalue  name TRAP  ;  signal on syntax name TRAP
   signal on failure  name TRAP  ;  signal on halt   name TRAP

   do while TEXT <> ''                 /* get rid of HTTP header  */
      parse var TEXT MORE (CRLF) TEXT
   end

   HOSTLIST = translate( HOSTLIST,, ',' )
   TEXT = translate( MORE,, left( CRLF, 1 ))
   EOL  = right( CRLF, 1 )
   do N = 1 while TEXT <> ''           /* replace bare LF by CRLF */
      parse var TEXT MORE (EOL) TEXT
      MORE = MORE word( HOSTLIST, N )
      call RXLOG 1, word( COPYLEFT, 1 ) || ':' MORE
      HOSTNAME = HOSTNAME || MORE || CRLF
   end N
   parse source . MORE .   ;  if MORE = 'COMMAND' then say HOSTNAME
   return HOSTNAME

ABORT:   procedure expose (EXPO) /* show a socket error on stderr */
   if TSOCK < 0 then return USAGE( arg( 1 ))
   NN = '(error' max( 0, SockSock_errno() - 10000 ) || ')'
   call SockPSock_errno NN       /* REXX handles missing function */
   call SockClose TSOCK          /* as syntax TRAP exiting anyway */
   return arg( 1 ) NN            /* caller can show NN in PM win. */

USAGE:   procedure expose (EXPO) /* show usage message and return */
   parse source . NN N           /* return if error & not COMMAND */
   if NN <> 'COMMAND' & arg( 1 ) <> '' then return arg( 1 )

   N = substr( N, 1 + lastpos( '/', translate( N, '/', '\' )))
   NN = CRLF arg( 1 ) CRLF CRLF  ;  if arg( 1 ) = '' then NN = CRLF
   NN = NN 'usage:' N 'interface hosts login password [param]' CRLF
   NN = NN ' e.g.:' N 'ppp0 test.dnsalias.org test test offline=YES'
   NN = NN CRLF ' e.g.:' N 'ppp0 test.dnsalias.org test test'
   NN = NN CRLF 'where multiple comma separated hosts are allowed.'
   NN = NN CRLF 'Separate parameters with "&" or ";" (not spaces),'
   NN = NN CRLF 'parameter wildcard=NOCHG enforced by' N || '.'
   say NN   ;  return NN         /* caller can show NN in PM win. */

BASIC:   procedure expose (EXPO) /* basic HTTP authorization line */
   if arg( 1 ) == '' then return ''
   SRC = x2b( c2x( arg( 1 ))) ;  ADD = ''
   DST =        'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdef'
   DST = DST || 'ghijklmnopqrstuvwxyz0123456789+/'
   do N = 0 until DST = ''    ;  parse var DST C.N 2 DST ;  end N
   do N = 1 to (( length( SRC ) // 24 ) / 4 ) // 3
      ADD = ADD || '='        ;  SRC = SRC || '00'
   end N
   do while SRC <> ''
      parse var SRC N 7 SRC   ;  N = x2d( b2x( N ))
      DST = DST || C.N
   end
   return CRLF || 'Authorization: Basic' DST || ADD

DYNIP:   procedure expose (EXPO) /* get dynamic IP of interface:  */
   QUEUE = RxQueue( 'Get' )   ;  N = queued()

   address CMD '@ifconfig' arg( 1 ) '2>&1 | RxQueue' QUEUE '/LIFO'

   select                        /* private QUEUE setup by caller */
      when queued() = N + 2 then do
         parse pull N   ;  pull  ;  parse var N 'inet ' IP .
         if verify( IP, '0.123456789' ) = 0 then return IP
      end
      when queued() = N + 1 then parse pull N
      when queued() < N + 1 then N = 'no result in RxQueue' QUEUE
      otherwise                  /* clear other ifconfig messages */
         do queued() -N ;  pull  ;  end
         N = 'unforeseen message or race condition RxQueue' QUEUE
   end
   exit ABORT( 'ifconfig' arg( 1 ) 'error:' N )

/* <URL:http://purl.net/xyzzy/src/rxsyslog.cmd>, (c) F. Ellermann */

RXLOG:   procedure expose (EXPO) /* send message to Syslog daemon */
   if RxFuncQuery( 'SockLoadFuncs' ) then do
      call RxFuncAdd 'SockLoadFuncs', 'RXSOCK', 'SockLoadFuncs'
      call SockLoadFuncs 'N'     /* TRAP if RXSOCK.DLL not found  */
   end

   parse arg PRI, MSG            /* OS/2 syslogd.exe on 127.0.0.1 */
   if right( word( MSG, 1 ), 1 ) <> ':' then do
      parse source . . MON    ;  MSG = MON || ':'  MSG
   end
   MSG = left( MSG, min( 2000, length( MSG )))

   select                        /* forcing REXX error if bad PRI */
      when 0 <= PRI & PRI < 8 then  PRI = '<' || PRI + 160 || '>'
      when PRI < 192          then  PRI = '<' || PRI + 0   || '>'
   end
   parse value date() with DAY MON .      ;  DAY = right( DAY, 2 )
   MSG = PRI || MON DAY time()      MSG   /* DAY format " D"/"DD" */

   return DGRAM( 127.0.0.1, 514, MSG, 'MSG_DONTROUTE' )

DGRAM:   procedure expose (EXPO) /* sendto host, port, msg, [flg] */
   if sign( verify( arg( 1 ), '0123456789.' )) = 0 then do
      call SockGetHostByAddr arg( 1 ), 'PEER.'
      if result = 0 then   PEER.ADDR = arg( 1 )
   end                           /* support IP without host name, */
   else do                       /* but not host name without IP: */
      call SockGetHostByName arg( 1 ), 'PEER.'
      if result = 0  then  return 'unknown' arg( 1 ) '[' h_errno ']'
   end

   PEER.PORT = arg( 2 ) ;  PEER.FAMILY = 'AF_INET'
   SOCK = SockSocket( PEER.FAMILY, 'SOCK_DGRAM', 'IPPROTO_UDP' )
   if SOCK < 0 then return 'socket error' errno

   if arg( 4, 'e' )
      then SENT = SockSendTo( SOCK, arg( 3 ), arg( 4 ),  'PEER.' )
      else SENT = SockSendTo( SOCK, arg( 3 ),            'PEER.' )

   if SENT < 0 then  SENT = 'socket error' errno   ;  else do
      SENT = length( arg( 3 )) - SENT
      if SENT > 0 then  SENT = 'lost' SENT 'bytes' ;  else SENT = ''
   end
   call SockClose SOCK  ;  return SENT

/* -------------------------------------------------------------- */

FATAL:                           /* except from HALT a script bug */
   signal on novalue  name TRAP  ;  signal on syntax name TRAP
   signal on failure  name TRAP  ;  signal on halt   name TRAP

   TRAP = SockClose( TSOCK )     /* close socket and handle TRAP: */
   if TRAP <> 0 then TRAP = SockPSock_errno( 'SockClose() =' TRAP )

/* see <URL:http://purl.net/xyzzy/rexxtrap.htm>, (c) F. Ellermann */

TRAP:                            /* select REXX exception handler */
   call trace 'O' ;  trace N           /* don't trace interactive */
   parse source TRAP                   /* source on separate line */
   TRAP = x2c( 0D ) || right( '+++', 10 ) TRAP || x2c( 0D0A )
   TRAP = TRAP || right( '+++', 10 )   /* = standard trace prefix */
   TRAP = TRAP strip( condition( 'c' ) 'trap:' condition( 'd' ))
   select
      when wordpos( condition( 'c' ), 'ERROR FAILURE' ) > 0 then do
         if condition( 'd' ) > ''      /* need an additional line */
            then TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
         TRAP = TRAP '(RC' rc || ')'   /* any system error codes  */
         if condition( 'c' ) = 'FAILURE' then rc = -3
      end
      when wordpos( condition( 'c' ), 'HALT SYNTAX'   ) > 0 then do
         if condition( 'c' ) = 'HALT' then rc = 4
         if condition( 'd' ) > '' & condition( 'd' ) <> rc then do
            if condition( 'd' ) <> errortext( rc ) then do
               TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
               TRAP = TRAP errortext( rc )
            end                        /* future condition( 'd' ) */
         end                           /* may use errortext( rc ) */
         else  TRAP = TRAP errortext( rc )
         rc = -rc                      /* rc < 0: REXX error code */
      end
      when condition( 'c' ) = 'NOVALUE'  then rc = -2 /* dubious  */
      when condition( 'c' ) = 'NOTREADY' then rc = -1 /* dubious  */
      otherwise                        /* force non-zero whole rc */
         if datatype( value( 'RC' ), 'W' ) = 0 then rc = 1
         if rc = 0                             then rc = 1
         if condition() = '' then TRAP = TRAP arg( 1 )
   end                                 /* direct: TRAP( message ) */

   TRAP = TRAP || x2c( 0D0A ) || format( sigl, 6 )
   signal on syntax name TRAP.SIGL     /* throw syntax error 3... */
   if 0 < sigl & sigl <= sourceline()  /* if no handle for source */
      then TRAP = TRAP '*-*' strip( sourceline( sigl ))
      else TRAP = TRAP '+++ (source line unavailable)'
TRAP.SIGL:                             /* ...catch syntax error 3 */
   if abbrev( right( TRAP, 2 + 6 ), x2c( 0D0A )) then do
      TRAP = TRAP '+++ (source line unreadable)'   ;  rc = -rc
   end
   select
      when 0 then do                   /* in pipes STDERR: output */
         parse version TRAP.REXX . .   /* REXX/Personal: \dev\con */
         signal on syntax name TRAP.FAIL
         if TRAP.REXX = 'REXXSAA'      /* fails if no more handle */
            then call lineout 'STDERR'  , TRAP
            else call lineout '\dev\con', TRAP
      end
      when 1 then do                   /* OS/2 PM: RxMessageBox() */
         signal on syntax name TRAP.FAIL
         call RxMessageBox ,           /* fails if not in PMREXX  */
            translate( TRAP, ' ', x2c( 0D )), , 'CANCEL', 'WARNING'
      end                              /* replace any CR by blank */
      otherwise   say TRAP ; trace ?L  /* interactive Label trace */
   end

   if condition() = 'SIGNAL' then signal TRAP.EXIT
TRAP.CALL:  return rc                  /* continue after CALL ON  */
TRAP.FAIL:  say TRAP ;  rc = 0 - rc    /* force TRAP error output */
TRAP.EXIT:  exit   rc                  /* exit for any SIGNAL ON  */
