/* REXX: watch etc/ppp0.pid and if this file is changed determine */
/*       actual dynamic IP.  If IP is new update DynDNS host with */
/*       RXDYNDNS IFCONFIG, host(s), USERNAME, PASSWORD, VARIABLE */
/*       Please edit the variables below for your DynDNS host(s), */
/*       separate multiple hosts by a comma in HOSTNAME.          */

/* Usage: WATCHPPP seconds|0|-|*                                  */
/* where seconds (e.g. 10) is a poll interval used to check file  */
/* mptn/etc/ppp0.pid for changes.  Use WATCHPPP * to (re)create a */
/* WPS object for WATCHPPP 10.  Maybe add an icon to this object, */
/* and/or put a shadow in the startup folder <WP_START>.          */

/* Use WATCHPPP - to mark the host(s) as offline (only donators), */
/* Use WATCHPPP 0 to terminate a running daemon in a PM session,  */
/* Use WATCHPPP -seconds to see a message for all updates.        */
/* Use WATCHPPP +seconds to see only update problems.             */
/* Please note that WATCHPPP waits after update / error messages, */
/* that's NOT what you want if your server should run unattended. */
/* The poll cycle is restricted by MAXSLEEP = 100 seconds, that's */
/* also the maximal delay to terminate a daemon with WATCHPPP 0.  */

   IFCONFIG = 'ppp0'             /* interface ppp0 using ppp0.pid */
   HOSTNAME =  'test.dnsalias.org'
   USERNAME =  'test'            /* DynDNS login for RXDYNDNS.CMD */
   PASSWORD =  'test'            /* DynDNS password               */
   VARIABLE = 'system=dyndns'    /* dummy, edit this if necessary */

/* <URL:http://purl.net/xyzzy/rxdyndns.htm>, (c) 2003 F.Ellermann */

   signal on novalue  name TRAP  ;  signal on syntax name TRAP
   signal on failure  name TRAP  ;  signal on halt   name TRAP
   parse source . . PATH         ;  MAXSLEEP = 100
   MYQ  = translate( IFCONFIG || '.daemon' )
   ETC  = XENV( 'ETC' ) || '\' || IFCONFIG || '.pid'
   CRLF = x2c( 0D0A )            ;  EXPO = 'CRLF MYQ MAXSLEEP'

   if arg( 1 ) = '-' then do
      if VARIABLE = ''  then  VARIABLE = 'offline=YES'
                        else  VARIABLE = 'offline=YES&' ||  VARIABLE
      call RXDYNDNS IFCONFIG, HOSTNAME, USERNAME, PASSWORD, VARIABLE
      exit WAIT( 'RXDYNDNS offline result:' CRLF || result )
   end
   if arg( 1 ) = '*' then do
      exit WAIT( 'object' MAKE( MYQ, 10 ) 'updated, have fun.' )
   end
   if datatype( arg( 1 ), 'W' ) = 0 then do
      call UTIL 'SysSearchPath'
      NIP = SysSearchPath( 'PATH', 'RXDYNDNS.CMD' )
      NIP = NIP IFCONFIG 'host(list)' USERNAME PASSWORD VARIABLE
      NEW =            ' usage:' PATH '[seconds|0|-|*]'
      NEW = NEW CRLF 'checks' ETC 'cyclically with the given'
      NEW = NEW 'delay.' CRLF 'If changed and IFCONFIG' IFCONFIG
      NEW = NEW 'finds a new IP,'
      NEW = NEW 'then host(list)' CRLF HOSTNAME
      NEW = NEW CRLF 'is updated with the configured parameters:'
      NEW = NEW CRLF || NIP  || CRLF
      NEW = NEW CRLF 'Specify seconds < 0 to get a message even'
      NEW = NEW 'for GOOD updates,'
      NEW = NEW CRLF 'use "' || PATH '0" to kill a running'
      NEW = NEW MYQ  || ','
      NEW = NEW CRLF 'use "' || PATH '-" for offline=YES'
      NEW = NEW CRLF 'use "' || PATH '*" to create a'
      NEW = NEW MYQ 'object. '
      exit WAIT( NEW )
   end
   if abs( arg( 1 )) > MAXSLEEP then do
      exit WAIT( 'not supported:' abs( arg( 1 )) '>' MAXSLEEP )
   end

   if ONOFF() then exit QTRAP( 'cannot terminate running' MYQ )
   signal on novalue  name QTRAP ;  signal on syntax name QTRAP
   signal on failure  name QTRAP ;  signal on halt   name QTRAP
   OIP = 0.0.0.0                 ;  OLD = ''

   if arg( 1 ) <> 0 then do until SLEEP( arg( 1 ))
      NEW = stream( ETC, 'C', 'QUERY DATETIME' )
      if OLD = NEW   then  iterate     /* known OLD ppp session   */

      NIP = GETIP( IFCONFIG )          /* new session: get new IP */
      if OIP =  NIP  then  iterate     /* still the same known IP */
      if  '' =  NIP  then  iterate     /* new IP not yet known    */
      OLD = NEW

      call RXDYNDNS IFCONFIG, HOSTNAME, USERNAME, PASSWORD, VARIABLE

      if arg( 1 ) > 0 then do          /* else show each update   */
         TXT = translate( result )
         do while abbrev( TXT, 'GOOD' )
            parse var TXT (CRLF) TXT   /* get rid of GOOD updates */
         end
         if TXT = '' then iterate      /* if all is GOOD continue */
      end
      if WAIT( 'RXDYNDNS update result:' CRLF || result ) = 0
         then leave
   end
   exit ONOFF() | WAIT( MYQ 'terminated' )

ONOFF:   procedure expose (EXPO) /* manage private daemon queue   */
   QUE = RxQueue( 'Get' )
   if QUE = MYQ then do
      call RxQueue 'Delete', MYQ ;  if result = 0  then  return 0
   end
   else do MAXSLEEP until QUE = MYQ
      QUE = RxQueue( 'Create', MYQ )
      call RxQueue 'Set'   , MYQ ;  if QUE = MYQ   then  return 0
      call RxQueue 'Delete', QUE ;  if result <> 0 then  leave
      if queued() = 0 then queue 'termination request' date() time()
      call SLEEP 1
   end
   exit QTRAP( 'fatal: delete' QUE 'result' result )

SLEEP:   procedure expose (EXPO) /* sleep some seconds...         */
   signal on syntax name SLEEP.1 /* catch missing function error  */
   call SysSleep abs( arg( 1 ))  ;  return sign( queued())
SLEEP.1:                         /* 0: okay, 1: queue not empty   */
   signal on syntax name QTRAP   ;  call UTIL 'SysSleep'
   call SysSleep abs( arg( 1 ))  ;  return sign( queued())

GETIP:   procedure expose (EXPO) /* get dynamic IP of interface:  */
   signal on error name QTRAP ;  if queued() > 0 then return ''
   address CMD '@ifconfig' arg( 1 ) '2>&1 | RxQueue' MYQ '/LIFO'
   select
      when queued() = 0 then N = 'no result in RxQueue' MYQ
      when queued() = 2 then do
         parse pull N   ;  pull  ;  parse var N 'inet ' IP .
         if verify( IP, '0.123456789' ) = 0 then return IP
      end
      when queued() = 1 then do  /* return nothing if offline     */
         parse pull N   ;  return ''
      end
      otherwise                  /* clear other ifconfig messages */
         do queued()    ;  pull  ;  end
         N = 'unforeseen message or race condition RxQueue' MYQ
   end
   exit QTRAP( 'ifconfig' arg( 1 ) 'error:' N )

/* see <URL:http://purl.net/xyzzy/rexxtrap.htm>, (c) F. Ellermann */

XENV: procedure                  /* DOS REXX portable environment */
   parse version ENV . .
   if ENV = 'REXXSAA' then do
      parse source ENV . .       /* OS/2 REXXSAA:  os2environment */
      if ENV = 'OS/2'   then ENV = 'OS2ENVIRONMENT'
                        else ENV = ENV || 'ENVIRONMENT'
   end                           /* DOS  REXXSAA:  DOSENVIRONMENT */
   else  ENV = 'ENVIRONMENT'     /* REXX/Personal:    environment */
   select
      when arg() = 1 then  return value( arg( 1 ),/* get */, ENV )
      when arg() = 2 then  return value( arg( 1 ), arg( 2 ), ENV )
      otherwise            return abs( /* force REXX error 40 */ )
   end

WAIT: procedure                  /* get OKay resp. CANCEL answer: */
   KEY = PROC()                  ;  OUT = 'STDERR'

   select
      when KEY = 1 then do       /* 1 (real) obsolete: here DOS   */
         parse version KEY . .   /* REXX/Personal has no STDERR:  */
         if KEY <> 'REXXSAA' then OUT = '\dev\con'
      end
      when KEY = 3 then do       /* 3 (PM) RxMessageBox() output  */
         parse source KEY  ;  KEY = centre( KEY, 100 )   /* HACK  */
         KEY = RxMessageBox( arg( 1 ), KEY, 'OKCANCEL', 'ASTERISK' )
         return KEY = 1 | KEY = 6 | KEY = 8
      end                        /* 0 (fullscreen) and 2 (window) */
      when KEY < 4 then call UTIL 'SysGetKey'
      otherwise nop              /* 4 (detached) tested in AKEY() */
   end
   call charout OUT, arg( 1 ) || x2c( 7 )

   do until c2d( KEY ) <> 0 & c2d( KEY ) <> 224
      KEY = AKEY()
   end
   call lineout OUT, ''          /* hardwiring F3 '=', Alt-F4 'k' */
   return KEY <> x2c( 1B ) & KEY <> '=' & KEY <> 'k'

AKEY: procedure                  /* keyboard char. input function */
   KEY = PROC()
   if KEY == 4          then return x2c( 1B )      /* 4: detached */
   if KEY <> 1          then return SysGetKey( 'NoEcho' )
   parse version KEY . .                           /* 1: DOS REXX */
   if KEY == 'REXXSAA'  then return  RxGetKey( 'NoEcho' )
                        else return right( INKEY(), 1 )

PROC: procedure                  /* avoid "unknown function" TRAP */
   parse source OS .             /* for REXXSAA portability abuse */
   if OS <> 'OS/2' then return 1 /* the now obsolete 1: real mode */

   OS = 'ProcessType'            /* assume Sys... = RxProcessType */
   if RxFuncQuery( 'Sys' || OS ) = 0 then return SysProcessType()
   if RxFuncAdd(   'Sys' || OS, 'RxUtils', 'Rx' || OS ) = 0 then do
      signal on syntax name PROC.TRAP  ;  return SysProcessType()
   end                           /* tries RxUtils only once, else */
PROC.TRAP:                       /* force RexxUtil SysProcessType */
   call  RxFuncDrop 'SysProcessType'   ;  signal on syntax name TRAP
   call        UTIL 'SysProcessType'   ;  return SysProcessType()

MAKE: procedure                  /* recreate or update WPS object */
   /* 1st arg: optional object title, default name of source      */
   /* 2nd arg: optional start arg.s,  use '[txt]' if interactive  */
   /* 3rd arg: optional start directory, default TMP environment  */
   TMP = value( 'TMP',, 'OS2ENVIRONMENT' )
   if TMP = '' then TMP = directory()
   call UTIL 'SysCreateObject'      ;  call UTIL 'SysGetEA'
   parse upper source . . SRC       ;  POS = lastpos( '\', SRC )
   TXT = substr( SRC, POS + 1 )     ;  DIR = left( SRC, POS )
   OBJ = '<' || TXT || '>'          ;  POS = lastpos( '.', TXT )
   TXT = left( TXT, POS - 1 )       ;  NEW = 0

   ICO = stream( DIR || TXT || '.ICO', 'c', 'query exists' )
   if ICO = '' & SysGetEA( SRC, '.ICON', 'POS' ) = 0 then do
      call UTIL 'SysTempFileName'   ;  call UTIL 'SysFileDelete'
      signal on notready name TRAP  ;  SET = substr( POS, 5 )
      NEW = ( length( SET ) = c2d( reverse( substr( POS, 3, 2 ))))
      NEW = NEW & abbrev( POS, x2c( 'F9FF' ))
      if NEW then do             /* SysTempFileName error ignored */
         ICO = SysTempFileName( TMP || '\TMP?????.ICO' )
         call charout ICO, SET      ;  call charout ICO
      end
   end

   if arg( 3, 'O' ) then DIR = TMP  ;  else DIR = arg( 3 )
   if arg( 1, 'E' ) then TXT = arg( 1 )

   SET = 'EXENAME=*;PARAMETERS=/C' strip( SRC arg( 2 ))
   SET = SET || ';MINIMIZED=YES;PROGTYPE=PM'
   if ICO <> '' then SET = SET || ';ICONFILE=' || ICO
   SET = SET || ';STARTUPDIR=' || DIR || ';OBJECTID=' || OBJ || ';'

   POS = '<WP_DESKTOP>'
   POS = SysCreateObject( 'WPProgram', TXT, POS, SET, 'Update' )
   if NEW then call SysFileDelete ICO
   if POS then return OBJ        /* ready to create shadow etc.   */
          else exit TRAP( 'fatal - cannot update' OBJ )

UTIL: procedure                  /* load necessary RexxUtil entry */
   if RxFuncQuery(  arg( 1 )) then
      if RxFuncAdd( arg( 1 ), 'RexxUtil', arg( 1 )) then
         exit TRAP( "can't add RexxUtil"  arg( 1 ))
   return 0

QTRAP:   TRAP = RxQueue( 'Delete', RxQueue( 'Get' ))
TRAP:                            /* select REXX exception handler */
   call trace 'O' ;  trace N           /* don't trace interactive */
   parse source TRAP                   /* source on separate line */
   TRAP = x2c( 0D ) || right( '+++', 10 ) TRAP || x2c( 0D0A )
   TRAP = TRAP || right( '+++', 10 )   /* = standard trace prefix */
   TRAP = TRAP strip( condition( 'c' ) 'trap:' condition( 'd' ))
   select
      when wordpos( condition( 'c' ), 'ERROR FAILURE' ) > 0 then do
         if condition( 'd' ) > ''      /* need an additional line */
            then TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
         TRAP = TRAP '(RC' rc || ')'   /* any system error codes  */
         if condition( 'c' ) = 'FAILURE' then rc = -3
      end
      when wordpos( condition( 'c' ), 'HALT SYNTAX'   ) > 0 then do
         if condition( 'c' ) = 'HALT' then rc = 4
         if condition( 'd' ) > '' & condition( 'd' ) <> rc then do
            if condition( 'd' ) <> errortext( rc ) then do
               TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
               TRAP = TRAP errortext( rc )
            end                        /* future condition( 'd' ) */
         end                           /* may use errortext( rc ) */
         else  TRAP = TRAP errortext( rc )
         rc = -rc                      /* rc < 0: REXX error code */
      end
      when condition( 'c' ) = 'NOVALUE'  then rc = -2 /* dubious  */
      when condition( 'c' ) = 'NOTREADY' then rc = -1 /* dubious  */
      otherwise                        /* force non-zero whole rc */
         if datatype( value( 'RC' ), 'W' ) = 0 then rc = 1
         if rc = 0                             then rc = 1
         if condition() = '' then TRAP = TRAP arg( 1 )
   end                                 /* direct: TRAP( message ) */

   TRAP = TRAP || x2c( 0D0A ) || format( sigl, 6 )
   signal on syntax name TRAP.SIGL     /* throw syntax error 3... */
   if 0 < sigl & sigl <= sourceline()  /* if no handle for source */
      then TRAP = TRAP '*-*' strip( sourceline( sigl ))
      else TRAP = TRAP '+++ (source line unavailable)'
TRAP.SIGL:                             /* ...catch syntax error 3 */
   if abbrev( right( TRAP, 2 + 6 ), x2c( 0D0A )) then do
      TRAP = TRAP '+++ (source line unreadable)'   ;  rc = -rc
   end
   select
      when 0 then do                   /* in pipes STDERR: output */
         parse version TRAP.REXX . .   /* REXX/Personal: \dev\con */
         signal on syntax name TRAP.FAIL
         if TRAP.REXX = 'REXXSAA'      /* fails if no more handle */
            then call lineout 'STDERR'  , TRAP
            else call lineout '\dev\con', TRAP
      end
      when 1 then do                   /* OS/2 PM: RxMessageBox() */
         signal on syntax name TRAP.FAIL
         call RxMessageBox ,           /* fails if not in PMREXX  */
            translate( TRAP, ' ', x2c( 0D )), , 'CANCEL', 'WARNING'
      end                              /* replace any CR by blank */
      otherwise   say TRAP ; trace ?L  /* interactive Label trace */
   end

   if condition() = 'SIGNAL' then signal TRAP.EXIT
TRAP.CALL:  return rc                  /* continue after CALL ON  */
TRAP.FAIL:  say TRAP ;  rc = 0 - rc    /* force TRAP error output */
TRAP.EXIT:  exit   rc                  /* exit for any SIGNAL ON  */
